#!/usr/bin/env perl
use strict;


my $param_out = undef;
my $param_id = "double";
my $param_name = "Double";
my $param_factiondir = ".";
my @param_factions = ();


my $param = undef;
while (@ARGV) {
	my $arg = shift(@ARGV);
	if ($arg =~ m/^--/) {
		$param = $arg;
	} elsif ("--out" eq $param) {
		$param_out = $arg;
	} elsif ("--id" eq $param) {
		$param_id = $arg;
	} elsif ("--name" eq $param) {
		$param_name = $arg;
	} elsif ("--factiondir" eq $param) {
		$param_factiondir = $arg;
	} elsif ("--factions" eq $param) {
		push(@param_factions, $arg);
	} else {
		die("Unknown parameter: " . $param . "\n");
	}
}


my @factions = ();
foreach my $param_faction (@param_factions) {
	my $file_name = $param_factiondir . "/" . $param_faction;
	my $file;
	open($file, "<" . $file_name) or die("Cannot read file: " . $file_name . "\n");
	my $content = join("", <$file>);
	close($file);

	my $faction = {};
	if ($content =~ m/^\s*id=(.*)$/m) {
		$faction->{"id"} = $1;
	}
	if ($content =~ m/^\s*name=(.*)$/m) {
		$faction->{"name"} = $1;
	}
	if ($content =~ m/^\s*type=(.*)$/m) {
		$faction->{"type"} = $1;
	}
	if ($content =~ m/^\s*leader=(.*)$/m) {
		$faction->{"leader"} = $1;
		$faction->{"random_leader"} = $1;
	}
	if ($content =~ m/^\s*random_leader=(.*)$/m) {
		$faction->{"random_leader"} = $1;
	}
	if ($content =~ m/^\s*recruit=(.*)$/m) {
		$faction->{"recruit"} = $1;
	}
	if ($content =~ m/^\s*terrain_liked=(.*)$/m) {
		$faction->{"terrain_liked"} = $1;
	}
	if ($content =~ m/^\s*recruitment_pattern=(.*)$/m) {
		$faction->{"recruitment_pattern"} = $1;
	}
	push(@factions, $faction);
}


sub join_name {
	my $name1 = shift;
	my $name2 = shift;

	$name2 =~ m/"([^"]*)"[^"]*$/;
	my $n = $1;
	$name1 =~ s/("[^"]*)$/ and $n$1/;
	return $name1;
}


sub join_unit {
	my $unit1 = shift;
	my $unit2 = shift;

	my @us = split(/,/, $unit1);
	foreach my $u2 (split(/,/, $unit2)) {
		if (!grep { $u2 eq $_ } @us) {
			push(@us, $u2);
		}
	}
	return join(",", @us);
}


sub join_terrain {
	my $terrain1 = shift;
	my $terrain2 = shift;

	my @ts = split(//, $terrain1);
	foreach my $t2 (split(//, $terrain2)) {
		if (!grep { $t2 eq $_ } @ts) {
			push(@ts, $t2);
		}
	}
	return join("", @ts);
}


my $file;
open($file, ">" . $param_out) or die("Cannot write file: " . $param_out . "\n");
print $file "[era]\n";
print $file "id=" . $param_id . "\n";
print $file "name=_\"" . $param_name . "\"\n";
print $file "\n{RANDOM_SIDE}\n\n";

for (my $f1 = 0; $f1 < @factions; $f1++) {
 for (my $f2 = $f1 + 1; $f2 < @factions; $f2++) {
	my $faction1 = $factions[$f1];
	my $faction2 = $factions[$f2];

	print $file "[multiplayer_side]\n";
	print $file "id=" . $faction1->{"id"} . "_and_" . $faction2->{"id"} . "\n";
	print $file "name=" . join_name($faction1->{"name"}, $faction2->{"name"}) . "\n";
	print $file "type=" . $faction1->{"type"} . "\n";
	print $file "leader=" . join_unit($faction1->{"leader"}, $faction2->{"leader"}) . "\n";
	if (($faction1->{"leader"} ne $faction1->{"random_leader"}) || ($faction2->{"leader"} ne $faction2->{"random_leader"})) {
		print $file "random_leader=" . join_unit($faction1->{"random_leader"}, $faction2->{"random_leader"}) . "\n";
	}
	print $file "recruit=" . join_unit($faction1->{"recruit"}, $faction2->{"recruit"}) . "\n";
	print $file "terrain_liked=" . join_terrain($faction1->{"terrain_liked"}, $faction2->{"terrain_liked"}) . "\n";
	print $file "[ai]\n";
	print $file "recruitment_pattern=" . $faction1->{"recruitment_pattern"} . "," . $faction2->{"recruitment_pattern"} . "\n";
	print $file "[/ai]\n";
	print $file "[/multiplayer_side]\n\n";

 }
}

print $file "[/era]

\n";
close($file);
